
--
--  TjDragIt.lua
--    by Tuhljin
--
--  A quick way to make given frames draggable by the player.
--
--  TjDragIt.EnableDragging( myFrame[, child1[, child2[, ... childN]]] )
--    myFrame
--      The frame to be made draggable.
--    child1, child2, ... childN
--      A number of other frames that, when the player tries to drag them, should cause myFrame to move.
--
--  TjDragIt.EnableIndirectDragging( myFrame[, child1[, child2[, ... childN]]] )
--    As TjDragIt.EnableDragging except that it does not set myFrame to be dragged directly, instead relying on
--    the specified child frames to move it.
--
--  TjDragIt.EnablePositionSaving( myFrame, saveTable[, loadNow[, loadOnShow]] )
--    myFrame
--      A frame which has been or will be used as the myFrame argument of EnableDragging or EnableIndirectDragging.
--    saveTable
--      A table which TjDragIt will use to save the position of the frame when it has been dragged.
--    loadNow
--      If this evaluates to true, the frame will be moved to the position indicated by saveTable at this time.
--    loadOnShow
--      If this evaluates to true, the frame will be moved to the position indicated by saveTable whenever it is
--      shown. Useful for frames that would otherwise be automatically placed elsewhere when they are shown.
--
--  If you don't want parts of myFrame to be able to move off the screen, use myFrame:SetClampedToScreen(true).
--
--  Other functions:
--
--  TjDragIt.DisableDragging( myFrame )
--  TjDragIt.DisablePositionSaving( myFrame )
--


local THIS_VERSION = 0.3

if (not TjDragIt or TjDragIt.Version < THIS_VERSION) then
  TjDragIt = {}
  TjDragIt.Version = THIS_VERSION

  local parents, saveTabs = {}, {}

  local function savePosition(frame, tab)
    local obj
    tab.point, obj, tab.relativePoint, tab.x, tab.y = frame:GetPoint()
    tab.relativeTo = obj and obj:GetName() or "UIParent"
  end

  local function loadPosition_OnShow(frame)
    TjDragIt.LoadPosition(frame)
  end

  local function MouseDown(frame, button)
    local name = frame:GetName()
    frame = name and parents[name] or frame.TjDragIt_parent or frame
    if (frame:IsMovable() and button == "LeftButton") then
      frame:StartMoving()
    end
  end

  local function MouseUp(frame, button)
    local name = frame:GetName()
    frame = name and parents[name] or frame.TjDragIt_parent or frame
    if (button == "LeftButton") then
      frame:StopMovingOrSizing()
      name = frame:GetName()
      local tab = name and saveTabs[name] or frame.TjDragIt_saveTable
      if (tab) then  savePosition(frame, tab)  end
    end
  end

  local function sethook(frame, script, handler)
    local prev = frame:GetScript(script)
    if (prev) then
      frame:HookScript(script, handler)
    else
      frame:SetScript(script, handler)
    end
  end

  function TjDragIt.LoadPosition(frame, tab)
    local name = frame:GetName()
    local tab = tab or name and saveTabs[name] or frame.TjDragIt_saveTable
    assert( type(tab) == "table", "Position table not found: Needs to be given as 2nd argument or specified through TjDragIt.EnablePositionSaving()." )
    local point, relativeTo, relativePoint, x, y = tab.point, tab.relativeTo, tab.relativePoint, tab.x, tab.y
    if (point and relativeTo and relativePoint and x and y) then
      local obj = getglobal(relativeTo)
      assert(obj, "Could not find object: "..relativeTo)
      frame:ClearAllPoints()
      frame:SetPoint(point, obj, relativePoint, x, y)
    else
      savePosition(frame, tab)
    end
  end

  function TjDragIt.EnableIndirectDragging(frame, ...)
    frame:SetMovable(true)
    local child, name
    for i=1, select("#", ...) do
      child = select(i, ...)
      child:SetMovable(true)
      child:SetClampedToScreen(false)
      child:EnableMouse(true)
      if (not child.TjDragIt_hooked) then
        sethook(child, "OnMouseDown", MouseDown)
        sethook(child, "OnMouseUp", MouseUp)
        child.TjDragIt_hooked = true
      end
      name = child:GetName()
      if (name) then
        parents[name] = frame
      else
        child.TjDragIt_parent = frame
      end
    end
  end

  function TjDragIt.EnableDragging(frame, ...)
    TjDragIt.EnableIndirectDragging(frame, ...)
    frame:EnableMouse(true)
    if (not frame.TjDragIt_hooked) then
      sethook(frame, "OnMouseDown", MouseDown)
      sethook(frame, "OnMouseUp", MouseUp)
      frame.TjDragIt_hooked = true
    end
  end

  function TjDragIt.DisableDragging(frame, ...)
    frame:SetMovable(false)
    local child, name
    for i=1, select("#", ...) do
      child = select(i, ...)
      child:SetMovable(false)
    end
  end

  function TjDragIt.EnablePositionSaving(frame, saveTable, loadNow, loadOnShow)
    assert( type(saveTable) == "table", "Invalid argument #2: Expected table." )
    local name = frame:GetName()
    if (name) then
      saveTabs[name] = saveTable
    else
      frame.TjDragIt_saveTable = saveTable
    end
    if (loadNow) then
      TjDragIt.LoadPosition(frame, saveTable)
    else
      savePosition(frame, saveTable)
    end
    if (loadOnShow and not frame.TjDragIt_hookedOnShow) then
      sethook(frame, "OnShow", loadPosition_OnShow)
      frame.TjDragIt_hookedOnShow = true
    end
  end

  function TjDragIt.DisablePositionSaving(frame)
    frame.TjDragIt_saveTable = nil
    local name = frame:GetName()
    if (name) then  saveTabs[name] = nil; end
  end

end
